'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bindState = bindState;
exports.nameToPath = nameToPath;
exports.buildFormValidator = buildFormValidator;

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function bindState(component) {
  var key = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'form';

  return {
    attrs: component.state && component.state[key] || {},
    onChange: function onChange(attrs) {
      return component.setState(_defineProperty({}, key, attrs));
    }
  };
}

function nameToPath(name) {
  return name.replace(/\.(\d+)(\.)?/g, function (_match, i, dot) {
    return '[' + i + ']' + (dot ? '.' : '');
  });
}

function wildcard(name) {
  return name.replace(/\d+/g, '*');
}

function buildFormValidator(form) {
  function validate(name) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var value = options.hasOwnProperty('value') ? options.value : form.get(name);
    var validator = options['with'] || form._validations[name] || form._validations[wildcard(name)];

    if (!validator) return null;

    var error = callValidator(validator);

    if (error) {
      return validate.addError(name, error);
    }

    return null;

    function callValidator(validator) {
      if ((0, _isPlainObject2.default)(validator)) {
        return callObjectValidator(validator);
      }
      if ((0, _isArray2.default)(validator)) {
        return callArrayValidator(validator);
      }
      if (typeof validator === 'string') {
        return callStringValidator(validator);
      }
      if (typeof validator === 'function') {
        return validator.call(form, value);
      }
      throw new Error('unable to use \'' + validator + '\' as validator function');
    }

    function stringToValidator(name) {
      var validator = form.constructor.validations[name];

      if (!validator) throw new Error(name + ' validation rule is not defined');

      return validator;
    }

    function callStringValidator(name, options) {
      return stringToValidator(name).call(form, value, options === true ? {} : options);
    }

    function callObjectValidator(obj) {
      for (var _name in obj) {
        var _error = callStringValidator(_name, obj[_name]);
        if (_error) return _error;
      }
    }

    function callArrayValidator(ary) {
      for (var i = 0; i < ary.length; i++) {
        var _error2 = callValidator(ary[i]);
        if (_error2) return _error2;
      }
    }
  }

  Object.assign(validate, {
    errors: {},
    clearErrors: function clearErrors() {
      this.errors = {};
    },
    addError: function addError(key, message) {
      this.errors[key] = message;
      return message;
    },
    nested: function nested(ref) {
      var errors = form.refs[ref].performValidation();

      if (Object.getOwnPropertyNames(errors).length > 0) {
        this.addError(ref, 'invalid');
      }
    }
  });

  return validate;
}